import httpx
import json

from PyWdxServiceAgent.BizResponse import BizResponse
from PyWdxServiceAgent.WdxComputerInfo import WdxComputerInfo
from PyWdxServiceAgent.TypeEncoder import TypeEncoder

class BizServiceAgent:

    def __init__(self,useJwtToken = False,token=""):
        self.endpoint_url = ""
        self.useJwtToken = useJwtToken
        self.accessToken = "" if useJwtToken else token
        self.jwtToken = token if useJwtToken else ""

    def convert_default(obj):
        print(obj)
        if obj is None:
            return
        return obj
    
    def _ExecuteCore(self, request, operation):

        bizService_endpoint = self.endpoint_url + '/rest/bizservice'

        if self.useJwtToken == False:
            headers = {
                'Content-Type': 'application/json; charset=utf-8' ,
                'Accept':'appliction/json',
                'HHIRest-Authenticate' : self.accessToken}
        else:
            headers = {
                'Content-Type': 'application/json; charset=utf-8' ,
                'Accept':'appliction/json',
                'Authorization' : "Bearer " + self.jwtToken}

        if type(request).__name__ == "BizRequest":
            request.clientIP = WdxComputerInfo.IPAddress() if request.clientIP == "" else request.clientIP
            request.clientMachineName = WdxComputerInfo.MachineName() if request.clientMachineName == "" else request.clientMachineName
            request.clientMAC = WdxComputerInfo.MACAddress() if request.clientMAC == "" else request.clientMAC
        elif type(request).__name__ == "BizResponseCollection":
            for req in request.items:
                req.clientIP = WdxComputerInfo.IPAddress() if req.clientIP == "" else req.clientIP
                req.clientMachineName = WdxComputerInfo.MachineName() if req.clientMachineName == "" else req.clientMachineName
                req.clientMAC = WdxComputerInfo.MACAddress() if req.clientMAC == "" else req.clientMAC

        biz_request_json = json.dumps(request.to_dict(), default=self.convert_default, cls=TypeEncoder)

        print(biz_request_json)

        resp = httpx.post(bizService_endpoint + "/" + operation, headers=headers, data=biz_request_json)
        print(f"http code => {resp.status_code}")

        json_string = resp.content.decode('utf-8')

        data = json.loads(json_string)
        
        response = BizResponse()
        response.success = data.get('success')
        response.message = data.get('message')
        response.serverIP = data.get('serverIP')
        response.diagnostics = data.get('diagnostics')
        response.result = data.get('result')
        response.serviceLog = data.get('serviceLog')
        response.flags = data.get('flags')
        response.parameters = data.get('parameters')
        response.extraData = data.get('extraData')

        return response


    def Execute(self, request):
        return self._ExecuteCore(request=request, operation="execute")


    def ExecuteMultiple(self, requests):
        return self._ExecuteCore(request=requests, operation="executeMultiple")


import socket
import uuid
import platform

class WdxComputerInfo:

    @staticmethod
    def GetComputerInfo():
        if WdxComputerInfo._isGetComputerInfo == False:
            WdxComputerInfo._computer_name = socket.gethostname()
            WdxComputerInfo._ip_address = socket.gethostbyname(WdxComputerInfo._computer_name)
            WdxComputerInfo._mac_address = ''.join(['{:02x}'.format((uuid.getnode() >> elements) & 0xff)
                                for elements in range(0,8*6,8)][::-1]).upper()
            WdxComputerInfo._os_platform = platform.system()
            WdxComputerInfo._isGetComputerInfo = True

    _isGetComputerInfo = False
    _computer_name = ""
    _ip_address = ""
    _mac_address = ""
    _os_platform = ""

    @staticmethod
    def get_info():
        WdxComputerInfo.GetComputerInfo()
        return {
            "Computer Name": WdxComputerInfo.MachineName(),
            "IP Address": WdxComputerInfo.IPAddress(),
            "MAC Address": WdxComputerInfo.MACAddress(),
            "OS Platform": WdxComputerInfo.OSVersion()
        }
    
    @staticmethod
    def MachineName():
        WdxComputerInfo.GetComputerInfo()
        return WdxComputerInfo._computer_name

    @staticmethod
    def IPAddress():
        WdxComputerInfo.GetComputerInfo()
        return WdxComputerInfo._ip_address
    
    @staticmethod
    def MACAddress():
        WdxComputerInfo.GetComputerInfo()
        return WdxComputerInfo._mac_address

    @staticmethod
    def OSVersion():
        WdxComputerInfo.GetComputerInfo()
        return WdxComputerInfo._os_platform


class BizRequestFlags:
    _None = "None"
    Diagnostics = "Diagnostics"
    ServiceLog = "ServiceLog"


class BizRequest:

    def __init__(self):
        self.classId = ""
        self.methodId = ""
        self.clientIP = ""
        self.clientMachineName = ""
        self.clientMAC = ""
        self.serviceLogOption = False
        self.flags = BizRequestFlags._None
        self.parameters = {}
        self.enableLog = True
        self.executeId = ""
    
    def to_dict(self):
        return {
            'classId': self.classId,
            'methodId': self.methodId,
            'clientIP': self.clientIP,
            'clientMachineName': self.clientMachineName,
            'clientMAC': self.clientMAC,
            'serviceLogOption': self.serviceLogOption,
            'flags': self.flags,
            'parameters': self.parameters,
            'enableLog': self.enableLog,
            'executeId': self.executeId
        }

class BizRequestCollection():

    def __init__(self):
        self.serviceLogOption = False
        self.flags = BizRequestFlags._None
        self.items = []
        self.enableLog = True
        self.executeId = ""
        
    def to_dict(self):
        return {
            'serviceLogOption': self.serviceLogOption,
            'flags': self.flags,
            'items': [request.to_dict() for request in self.items],
            'enableLog': self.enableLog,
            'executeId': self.executeId
        }


import httpx
import json
import pandas as pd
import os

class WdxCertServiceAgent:

    def __init__(self,client_appid, userid, client_dns, service_appid, service_id):
        self.service_appid = service_appid;
        self.client_appid = client_appid;
        self.userid = userid;
        self.client_dns = client_dns;
        self.service_id = service_id;
        # .env 파일 로드(end point 주소를 가져오자!!)
        from dotenv import load_dotenv
        load_dotenv()

    def default(obj):
        print(obj)
        if obj is None:
            return
        return obj
    
    def GetJwtToken(self):

        endpoint_url = os.environ['WDX_JWTTOKEN_ENDPOINT']

        certService_endpoint = endpoint_url + '/GetSecurityKey'

        headers = {
            'Content-Type': 'application/json; charset=utf-8' ,
            'Accept':'appliction/json'
            }
        
        jwt_gen_request = { 
            "clientappid": self.client_appid,
            "userid": self.userid, 
            "clientdns": self.client_dns,
            "securitykey": "",
            "serviceid": self.service_id ,
            "appid": self.service_appid,
            }

        jwt_gen_request_json = json.dumps(jwt_gen_request, default=self.default)

        print(jwt_gen_request_json)

        resp = httpx.post(certService_endpoint, headers=headers, data=jwt_gen_request_json)

        print(f"http code => {resp.status_code}")

        json_string = resp.content.decode('utf-8')

        securityKey = json.loads(json_string)

        jwt_gen_request['securitykey'] = securityKey
        
        certService_endpoint = endpoint_url + '/GenerateToken'

        jwt_gen_request_json = json.dumps(jwt_gen_request, default=self.default)

        print(jwt_gen_request_json)

        resp = httpx.post(certService_endpoint, headers=headers, data=jwt_gen_request_json)

        print(f"http code => {resp.status_code}")

        data = resp.content.decode('utf-8').replace('"','')

        return data





from PyWdxServiceAgent.WdxUtil import WdxUtil
from PyWdxServiceAgent.WdxComputerInfo import WdxComputerInfo
from PyWdxServiceAgent.BizRequest import BizRequestFlags,BizRequest,BizRequestCollection
from PyWdxServiceAgent.BizResponse import BizResponse,BizResponseCollection
from PyWdxServiceAgent.BizServiceAgent import BizServiceAgent
from PyWdxServiceAgent.QueryParameterCollection import QueryParameterCollection
from PyWdxServiceAgent.QueryRequest import TransactionOption,ExecutionOption,QueryRequestFlags,QueryRequest,QueryRequestCollection
from PyWdxServiceAgent.QueryResponse import QueryResponse,QueryResponseCollection
from PyWdxServiceAgent.QueryServiceAgent import QueryServiceAgent
from PyWdxServiceAgent.WdxCertServiceAgent import WdxCertServiceAgent


class TransactionOption:
    TxLocal = "TxLocal"
    TxDist = "TxDist"
    TxNone = "TxNone"
    TxSerializable = "TxSerializable"

class ExecutionOption:
        ExecNone      = "None"
        '''
        요청이 ExecuteDataSet 메서드를 수행함
        '''
        ExecuteDataSet      = "ExecuteDataSet"
        '''
        요청이 ExecuteNonQuery 메서드를 수행함
        '''
        ExecuteNonQuery     = "ExecuteNonQuery"
        '''
        요청이 ExecuteScalar 메서드를 수행함
        '''
        ExecuteScalar       = "ExecuteScalar"

class QueryRequestFlags:
        FlagNone      = "None"
        '''
        진단 호출
        '''
        Diagnostics         = "Diagnostics",
        '''
        서버측 출력 메시지(오라클의 dbms_output 등) 반환하도록 요청
        '''
        DbmsOutput          = "DbmsOutput",
        '''
        서비스 측 로그 메시지를 반환하도록 요청
        '''
        ServiceLog          = "ServiceLog",

class QueryRequest:

    def __init__(self):
        self.mapperName = ""
        self.queryId = ""
        self.clientIP = ""
        self.clientMachineName = ""
        self.clientMAC = ""
        self.commandTimeout = 0
        self.transactionTimeout = 600
        self.transactionOption = TransactionOption.TxNone
        self.executionOption = ExecutionOption.ExecNone
        self.dbmsOutputOption = False
        self.serviceLogOption = False
        self.flags = QueryRequestFlags.FlagNone
        self.parameters = {}
        self.dataSet = None
        self.enableLog = True
        self.executeId = ""
    
    def to_dict(self):
        return {
            'mapperName': self.mapperName,
            'queryId': self.queryId,
            'clientIP': self.clientIP,
            'clientMachineName': self.clientMachineName,
            'clientMAC': self.clientMAC,
            'commandTimeout': self.commandTimeout,
            'transactionTimeout': self.transactionTimeout,
            'transactionOption': self.transactionOption,
            'executionOption': self.executionOption,
            'dbmsOutputOption': self.dbmsOutputOption,
            'serviceLogOption': self.serviceLogOption,
            'flags': self.flags,
            'parameters': self.parameters,
            'dataSet': self.dataSet,
            'enableLog': self.enableLog,
            'executeId': self.executeId
        }

class QueryRequestCollection():
    def __init__(self):
        self.mapperName = ""
        self.transactionOption = TransactionOption.TxNone
        self.transactionTimeout = 600
        self.serviceLogOption = False
        self.flags = QueryRequestFlags.FlagNone
        self.items = []
        self.enableLog = True
        self.executeId = ""

    def to_dict(self):
        return {
            'mapperName': self.mapperName,
            'transactionTimeout': self.transactionTimeout,
            'transactionOption': self.transactionOption,
            'serviceLogOption': self.serviceLogOption,
            'flags': self.flags,
            'items': [request.to_dict() for request in self.items],
            'enableLog': self.enableLog,
            'executeId': self.executeId
        }


from setuptools import setup, find_packages

setup(
    name="PyWdxServiceAgent",  # 패키지 이름
    version="0.1.17",  # 버전
    packages=find_packages(),  # 패키지들을 자동으로 찾음
    install_requires=[
        # 여기에 필요한 패키지 의존성을 작성
        # 예: 'numpy>=1.21.0', 'pandas'
    ],
    author="genfeel",
    author_email="genfeel@hotmail.com",
    description="Wdworker BackendService Agent",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url="https://github.com/genfeel/Wdworker",  # GitHub 리포지토리 URL
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",  # MIT 라이선스 (변경 가능)
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',  # 파이썬 최소 버전
)



import json

class TypeEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, int):
            return {"type": "int", "value": obj}
        elif isinstance(obj, float):
            return {"type": "float", "value": obj}
        elif isinstance(obj, str):
            return {"type": "str", "value": obj}
        elif isinstance(obj, bool):
            return {"type": "bool", "value": obj}
        elif obj is None:
            return {"type": "null", "value": None}
        else:
            # 기본적으로 처리할 수 없는 타입은 super로 처리
            return super().default(obj)


class QueryParameterCollection(dict):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.arrayBindCount(0)
        self.txCode([])
        self.txMessage("")
        self.fetchRowCount(0)
        self.txIsRollback(False)
        self.associativeArraySize(0)

    def arrayBindCount(self,count):
        self['$arrayBindCount'] = count

    def txCode(self,code):
        self['$txCode'] = code
        
    def txMessage(self,msg):
        self['$txMessage'] = msg
        
    def fetchRowCount(self,msg):
        self['$fetchRowCount'] = msg
        
    def txIsRollback(self,isRollback):
        self['$txIsRollback'] = isRollback
        
    def associativeArraySize(self,size):
        self['$associativeArraySize'] = size
